@@ -43,7 +43,7 @@
 #include "mp2.h"
 #include "shm_memory_tool.h"
 
-static const char *VERSION	 = "0.81";
+static const char *VERSION	 = "0.84";
 static const char *DESCRIPTION	 = "bit stream out to S/P-DIF of a sound card";
 static const char *MAINMENUENTRY = "Bitstreamout";
 static ctrl_t setup = {
@@ -75,7 +75,6 @@ private:
     bool mp2dec;
     char *SPDIFmute;
     unsigned long rtc;
-    virtual void clean(void);
 protected:
     int active;
     int mp2spdif;
@@ -84,6 +83,7 @@ public:
     cBitStreamOut(void);
     virtual ~cBitStreamOut(void);
     virtual bool Start(void);
+    virtual void Stop(void);
     virtual const char *Version(void);
     virtual const char *Description(void);
     virtual const char *CommandLineHelp(void);
@@ -155,21 +155,30 @@ cBitStreamOut::cBitStreamOut(void)
 
 cBitStreamOut::~cBitStreamOut(void)
 {
+#if defined(VDRVERSNUM) && VDRVERSNUM < 10320
     set_setup(CLEAR);
-    clean();
+    Stop();
+#endif
 }
 
-void cBitStreamOut::clean(void)
+void cBitStreamOut::Stop(void)
 {
-#ifdef VDR_DOES_CHECK_OBJECT
-    if (ReplayOutSPDif)
-	delete ReplayOutSPDif;
-    ReplayOutSPDif = NULL;
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10320
+    set_setup(CLEAR);
 #endif
+    if (ReplayOutSPDif) {
+	ReplayOutSPDif->Clear();
+#ifdef VDR_DOES_CHECK_OBJECT
+	delete ReplayOutSPDif;
+	ReplayOutSPDif = NULL;
+#endif
+    }
 
-    if (ChannelOutSPDif)
+    if (ChannelOutSPDif) {
+	ChannelOutSPDif->Clear();
 	delete ChannelOutSPDif;
-    ChannelOutSPDif = NULL;
+	ChannelOutSPDif = NULL;
+    }
 
     if (spdifDev)
 	spdifDev.Close();
@@ -260,7 +269,8 @@ bool cBitStreamOut::Start(void)
     clear_setup(CLEAR);
     return true;
 err:
-    clean();
+    set_setup(CLEAR);
+    Stop();
     return false;
 }
 
@@ -320,7 +330,7 @@ bool cBitStreamOut::SetupParse(const char *Name, const char *Value)
     else if (!strcasecmp(Name, "Device"))     setup.opt.device   = atoi(Value);
     else if (!strcasecmp(Name, "Delay"))      setup.opt.delay    = atoi(Value);
     else if (!strcasecmp(Name, "LiveDelay"))  setup.opt.ldelay   = atoi(Value);
-    else if (!strcasecmp(Name, "PCMinital"))  setup.opt.mdelay   = atoi(Value);
+    else if (!strcasecmp(Name, "PCMinit"))    setup.opt.mdelay   = atoi(Value);
     else if (!strcasecmp(Name, "Mp2offset"))  setup.opt.adelay   = atoi(Value);
     else if (!strcasecmp(Name, "IEC958"))     setup.opt.type     = atoi(Value);
     else if (!strcasecmp(Name, "VariableIO")) setup.opt.variable = atoi(Value);
@@ -485,7 +495,7 @@ void cDisplayMainMenu::Store(void)
 {
     SetupStore("Delay",      setup.opt.delay    = opt.delay);
     SetupStore("LiveDelay",  setup.opt.ldelay   = opt.ldelay);
-    SetupStore("PCMinital",  setup.opt.mdelay   = opt.mdelay);
+    SetupStore("PCMinit",    setup.opt.mdelay   = opt.mdelay);
     SetupStore("Mp2offset",  setup.opt.adelay   = opt.adelay);
     SetupStore("Active",     ((active)    ? true : false));
     SetupStore("Mp2Enable",  ((mp2enable) ? true : false));
