@@ -22,6 +22,8 @@
  * Copyright (C) 2003 Sven Goethel, <sven@jausoft.com>
  */
 
+#define USE_SWITCH_THREAD
+
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
@@ -47,22 +49,22 @@ uint_8  * cInStream::tsdata;
 cBounce * cInStream::bounce;
 
 cInStream::cInStream(int Pid, spdif *dev, ctrl_t &up, cBounce *bPtr)
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10300
-# if defined(VDRVERSNUM) && VDRVERSNUM > 10318
+#if defined(VDRVERSNUM) && VDRVERSNUM > 10318
 :cReceiver(0, -1, Pid),
-# else
-:cReceiver(0, -1, 1, Pid),
-# endif
- cThread("bso(instream): Forwarding bitstream"),
- ctr(), wait(), spdifDev(dev), setup(up)
 #else
-:cReceiver(0, -1, 1, Pid), cThread(), ctr(), wait(), spdifDev(dev), setup(up)
+:cReceiver(0, -1, 1, Pid),
 #endif
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10300
+ cThread("bso(instream): Forwarding bitstream"),
+#else
+ cThread(),
+#endif
+ ctrl(), wait(), spdifDev(dev), setup(up)
 {
     flags = 0;
-    ctr.Lock();
+    ctrl.Lock();
     stream = NULL;
-    ctr.Unlock();
+    ctrl.Unlock();
     Apid = Pid;
     ResetScan();
     bounce = bPtr;
@@ -73,15 +75,16 @@ cInStream::cInStream(int Pid, spdif *dev, ctrl_t &up, cBounce *bPtr)
 
 cInStream::~cInStream(void)
 {
-    if (test_and_set_flag(DETACHED))
-	Detach();
-    ActivateInStream(false);
-    bounce->flush();
-    ResetScan();
-    clear_flag(FAILED);
+    if (test_flag(ACTIVE))
+	Clear();
+    else {
+	bounce->flush();
+	ResetScan();
+	clear_flag(FAILED);
+    }
 }
 
-void cInStream::ActivateInStream(bool onoff)
+void cInStream::Activate(bool onoff)
 {
     if (test_setup(CLEAR))
 	onoff = false;
@@ -90,7 +93,7 @@ void cInStream::ActivateInStream(bool onoff)
 
     if (onoff) {
 	if (*spdifDev)
-	    goto out;		// An other has opened the S/P-DIF
+	    goto out;				// An other has opened the S/P-DIF
 	if (test_setup(RESET)) {
 	    clear_flag(FAILED);
 	    clear_setup(RESET);
@@ -98,6 +101,7 @@ void cInStream::ActivateInStream(bool onoff)
 	if (test_flag(FAILED) || test_flag(ACTIVE))
 	    goto out;
 	set_flag(ACTIVE);
+	clear_flag(STREAMING);			// Streaming only after receiving data
 	set_setup(LIVE);
 	bounce->flush();
 	Start();				// Warning: Start() sleeps 10ms after work
@@ -117,11 +121,10 @@ void cInStream::ActivateInStream(bool onoff)
 	if (test_flag(RUNNING) || Active()) {
 	    esyslog("INSTREAM: Forwarding bitstream thread was broken");
 	    wait.msec(10);			// More than 2ms
-	    ctr.Lock();
+	    ctrl.Lock();
 	    iec60958* curr = stream;
-	    ctr.Unlock();
+	    ctrl.Unlock();
 	    if (curr) {				// thread broken
-		LOCK_THREAD;
 		spdifDev->Close();
 		clear_flag(BOUNDARY);
 		curr->Reset();
@@ -132,9 +135,10 @@ void cInStream::ActivateInStream(bool onoff)
 
 	ResetScan();
 	clear_setup(LIVE);
-	ctr.Lock();
+	clear_flag(STREAMING);
+	ctrl.Lock();
 	stream = NULL;
-	ctr.Unlock();
+	ctrl.Unlock();
     }
 out:
     return;
@@ -143,18 +147,30 @@ out:
 void cInStream::Action(void)
 {
     set_flag(RUNNING);
-    ctr.Lock();
-    iec60958 *curr = stream;
-    ctr.Unlock();
+    iec60958 *curr = NULL;
 
     realtime("INSTREAM: ");
+
+    do {
+	if (!test_flag(ACTIVE))
+	    break;
+	if (StreamReady.Wait(100)) {		// Wait 100ms
+	    ctrl.Lock();
+	    curr = stream;
+	    ctrl.Unlock();
+	    if (curr) set_flag(STREAMING);	// Ready for streaming
+	}
+    } while (!curr);
+
+    if (!test_flag(ACTIVE))
+	goto out;
+
     if (!curr) {
 	esyslog("INSTREAM: no stream for spdif interface");
 	set_flag(FAILED);
 	goto out;
     } else {
-	LOCK_THREAD;
-	if (!spdifDev->Open(curr)) {
+	if (!spdifDev->Open(curr, this)) {
 	    esyslog("INSTREAM: can't open spdif interface");
 	    set_flag(FAILED);
 	    goto out;
@@ -171,7 +187,6 @@ void cInStream::Action(void)
 
 	if (test_setup(MUTE)) {
 	    if (!test_flag(WASMUTED)) {
-		LOCK_THREAD;
 		spdifDev->Clear();
 		clear_flag(BOUNDARY);
 		bounce->flush();
@@ -181,16 +196,13 @@ void cInStream::Action(void)
 	}
 
 	if (!spdifDev->Synchronize(bounce))	// Wait on bounce->signal()
-	    break;
+	    continue;				// No break on large delay gaps
 
-	if (!test_flag(ACTIVE)) {
-	    debug_chl("%s %d\n", __FUNCTION__, __LINE__);
+	if (!test_flag(ACTIVE))
 	    break;
-	}
 
 	if (test_setup(MUTE)) {
 	    if (!test_flag(WASMUTED)) {
-		LOCK_THREAD;
 		spdifDev->Clear();
 		clear_flag(BOUNDARY);
 		bounce->flush();
@@ -203,18 +215,16 @@ void cInStream::Action(void)
 	if ((len = bounce->fetch(tsdata, spdifDev->Available(TRANSFER_MEM))))
 	    spdifDev->Forward(tsdata, len, bounce);
     }
-    Lock();
-    spdifDev->Close();
+    spdifDev->Close(this);
     clear_flag(BOUNDARY);
     curr->Reset();
-    Unlock();
 #if defined(VDRVERSNUM) && VDRVERSNUM < 10300
     dsyslog("INSTREAM: Forwarding bitstream thread ended (pid=%d)", getpid());
 #endif
 out:
-    ctr.Lock();
+    ctrl.Lock();
     stream = NULL;
-    ctr.Unlock();
+    ctrl.Unlock();
     bounce->flush();
     bounce->leaveio();
     clear_flag(RUNNING);
@@ -227,12 +237,9 @@ void cInStream::Receive(uchar *b, int cnt)
     uint_8 start = 0;
     static uint_16 skip = 0;
 
-    if (test_flag(DETACHED))
-	goto out;
-
     if (!test_setup(ACTIVE)) {
 	if (test_flag(ACTIVE))
-	    ActivateInStream(false);
+	    Activate(false);
 	goto out;
     }
 
@@ -286,7 +293,7 @@ void cInStream::Receive(uchar *b, int cnt)
     }
 
     ptr = &b[off];
-    if (stream == NULL) {		// We edit the buffer contents for this case
+    if (stream == NULL) {		// We edit the buffer contents in this case
 	ptr = &scan[4];
 	memcpy(ptr, &b[off], TS_SIZE-off);
     }
@@ -313,9 +320,9 @@ inline bool cInStream::ScanTSforAudio(uint_8 *buf, const int cnt, const bool wak
 {
     const uint_8 *const tail = buf + cnt;
     bool ret = true, search;
-    ctr.Lock();
+    ctrl.Lock();
     iec60958 *curr = stream;
-    ctr.Unlock();
+    ctrl.Unlock();
 
 resync:
     if (bfound < bytes) {
@@ -325,6 +332,10 @@ resync:
 	//
 	switch (bfound) {
 	case 0 ... 3:
+
+	    if (!wakeup && !curr)			// No start TS frame, no PES start
+		goto reset;
+
 	    //
 	    // Scan for start sequence of the Private Stream 1 or Mpeg2 Audio
 	    //
@@ -514,38 +525,40 @@ resync:
 	    case 1:
 		buf -= 2;
 		buf[0] = 0x0b; buf[1] = 0x77;
+		suboff = 0;
 		curr = &ac3;
 		clear_setup(AUDIO);
 		curr->Reset(setup.flags);
 		curr->isDVD = false;
-		curr->track = 0x0b;
+		curr->track = 0x21;
 		audioType = audioTypes[IEC_AC3];
 		break;
 	    case 2:
 		buf -= 4;
 		buf[0] = 0x7f; buf[1] = 0xfe;
 		buf[2] = 0x80; buf[3] = 0x01;
+		suboff = 0;
 		curr = &dts;
 		clear_setup(AUDIO);
 		curr->Reset(setup.flags);
 		curr->isDVD = false;
-		curr->track = 0x7f;
+		curr->track = 0x21;
 		audioType = audioTypes[IEC_DTS];
 		break;
-	    case 3:				// DVD AC3 data stream may used over DVB (??)
+	    case 3:					// DVD AC3 data stream may used over DVB (??)
 		curr = &ac3;
 		clear_setup(AUDIO);
 		curr->Reset(setup.flags);
 		curr->isDVD = true;
-		curr->track = (paystart & 7) + 1;
+		curr->track = (paystart & 0x1F) + 0x21;
 		audioType = audioTypes[IEC_AC3];
 		break;
-	    case 4:				// DVD DTS data stream may used over DVB (??)
+	    case 4:					// DVD DTS data stream may used over DVB (??)
 		curr = &dts;
 		clear_setup(AUDIO);
 		curr->Reset(setup.flags);
 		curr->isDVD = true;
-		curr->track = (paystart & 7) + 1;
+		curr->track = (paystart & 0x1F) + 0x21;
 		audioType = audioTypes[IEC_DTS];
 		break;
 	    case 0:
@@ -556,7 +569,27 @@ resync:
 	} else {
 	    //
 	    // At BOUNDARY scanning for Mpeg2 Audio data start is not needed
+	    // we do it here to save some bytes in the bounce buffer
 	    //
+	    while (bfound + 2 <= paklen) {
+		if (buf >= tail)
+		    goto out;
+		submagic = (submagic << 8) | *buf;
+		buf++; subfnd++; bfound++;
+
+		if ((submagic & 0xfffe) == 0xffe0) {
+		    const bool ext  = ((submagic & 0x0010) >> 4) == 0;
+		    const bool lsf  = ((submagic & 0x0008) >> 3) == 0;
+		    const int layer = 4 - ((submagic & 0x0006) >> 1);
+
+		    if ((layer == 4) || (!lsf && ext))	// Sanity check for mpeg audio
+			continue;
+
+		    buf -= 2;
+		    ((uint_16*)&buf[0])[0] = ntohs(submagic);
+		    break;				// Found, skip rest
+		}
+	    }
 	    curr = &mp2;
 	    if (test_setup(MP2SPDIF))
 		clear_setup(AUDIO);
@@ -564,21 +597,19 @@ resync:
 		set_setup(AUDIO);
 	    curr->Reset(setup.flags);
 	    curr->isDVD = false;
-	    curr->track = (((uint_8)(syncword)) & 7) + 1;
+	    curr->track = ((uint_8)(syncword & 0xFF) - 0xC0) + 1;
 	    audioType = audioTypes[IEC_MP2];
 	}
-	ctr.Lock();
-	stream = curr;
-	ctr.Unlock();
+
     } else if (test_flag(PS1AUDIO) && curr && curr->isDVD) {
 	// Skip sub stream head for DVD case
 	suboff = 4;
 	switch (*buf) {
 	case 0x80 ... 0x87:
-	    if (curr == &ac3 && curr->track == (*buf & 7) + 1)
+	    if (curr == &ac3 && curr->track == ((*buf) & 0x1F) + 0x21)
 		break;
 	case 0x88 ... 0x8f:
-	    if (curr == &dts && curr->track == (*buf & 7) + 1)
+	    if (curr == &dts && curr->track == ((*buf) & 0x1F) + 0x21)
 		break;
 	default:
 	    goto reset;
@@ -598,13 +629,19 @@ resync:
     // Stamp (PTS). With this we should get as fast as
     // possible into A/V sync.
     //
-    if (!test_flag(RUNNING)) {
-	if (!wakeup || !test_flag(SET_PTS))
+    if (!test_flag(STREAMING)) {
+	if (!test_flag(SET_PTS))
 	    goto reset;
-	ActivateInStream(true);
+	//
+	// Set the stream we use and signal the awaiting thread
+	//
+	ctrl.Lock();
+	stream = curr;
+	ctrl.Unlock();
+	StreamReady.Signal(true);
 	clear_flag(WASMUTED);
     } else if (test_flag(WASMUTED)) {
-	if (!wakeup || !test_flag(SET_PTS))
+	if (!test_flag(SET_PTS))
 	    goto reset;
 	clear_flag(WASMUTED);
     }
@@ -670,8 +707,8 @@ resync:
     // Bytes found are equal to packet length if we have found the PES packet,
     // for the rest restart scanning.
     //
-    if (paklen && (bfound == paklen)) {
-	ResetScan(false);				// Reset for next scan
+    if (paklen && (bfound >= paklen)) {
+	ResetScan(false);			// Reset for next scan
 	if (buf >= tail)
 	    goto out;
 	goto resync;				// Check the rest for next frame
@@ -683,45 +720,86 @@ reset:
     return ret;
 }
 
+
+void cInStream::Clear(void)
+{
+    Activate(false);
+    bounce->flush();
+    ResetScan();
+    clear_flag(FAILED);
+}
+
 // --- cChannelOutSPDif : Live AC3 stream over S/P-DIF of a sound card with ALSA ---------
 cBounce * cChannelOutSPDif::bounce;
 const char * cChannelOutSPDif::audioType = audioTypes[IEC_NONE];
 uint_16 cChannelOutSPDif::Apid = 0x1FFF;
 
 cChannelOutSPDif::cChannelOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const char *script)
+:cStatus(),
 #if defined(VDRVERSNUM) && VDRVERSNUM >= 10300
-:cStatus(), cThread("bso(channelout): Switching bitstream"),
- Channel(NULL), wait(), SPDIFmute(script), spdifDev(&dev), setup(up)
+ cThread("bso(channelout): Switching bitstream"),
 #else
-:cStatus(), cThread(), Channel(NULL), wait(), SPDIFmute(script), spdifDev(&dev), setup(up)
+ cThread(),
 #endif
+ Channel(NULL), wait(), SPDIFmute(script), spdifDev(&dev), setup(up)
 {
+    flags = 0;
     in = NULL;
     replaying = false;
+    recording = false;
     bounce = bPtr;
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-    tracktimeout = 0;
-#else
+#if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
     Player = NULL;
-    Activate(true);
 #endif
 }
 
 cChannelOutSPDif::~cChannelOutSPDif(void)
 {
-    Activate(false);
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10320
+    if (test_flag(ACTIVE))
+#endif
+    Clear();
     if (in) AttachReceiver(false);
 }
 
+bool cChannelOutSPDif::InTransferMode(void)
+{
+    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
+
+    // Sanity check not be attached in transfer mode.
+    // In this case the replay part should take the data
+    if (PrimaryDevice != cDevice::ActualDevice())
+	goto out;
+
+    // Next sanity check not be attached in transfer mode
+    // from the primary device to the primary device.
+    // As metioned above the replay part should take the data
+    if (PrimaryDevice == cTransferControl::ReceiverDevice())
+	goto out;
+
+    return false;
+out:
+    sw.Lock();
+    if (in) AttachReceiver(false);
+    Channel = NULL;
+    sw.Unlock();
+    return true;
+}
+
 #if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-# define TRACKTIMEOUT	6000 // ms
 void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
 {
     const char* type = audioTypes[IEC_NONE];
     uint16_t apid = 0x1FFF;
     int num = 0;
 
-    if (replaying || !Channel)
+    if (replaying || recording)
+	goto out;
+
+    if (!Channel)
+	goto out;
+
+    if (InTransferMode())
 	goto out;
 
     if (!test_setup(ACTIVE))
@@ -745,15 +823,20 @@ void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
     if (Apid == apid)
 	goto out;
 
-    tracktimeout = TRACKTIMEOUT;
+    sw.Lock();
+    if (in) AttachReceiver(false);
     if (!test_flag(RUNNING)) {
 	clear_flag(ACTIVE);
 	Activate(true);
     }
+    sw.Unlock();
+    ctrl.Lock();
+    watch.Broadcast();
+    ctrl.Unlock();
 out:
     return;
 }
-#endif // VDRVERSNUM >= 10321
+#endif // VDRVERSNUM < 10321
 
 void cChannelOutSPDif::Activate(bool onoff)
 {
@@ -793,34 +876,33 @@ out:
 void cChannelOutSPDif::Action(void)
 {
     set_flag(RUNNING);
-# if defined(VDRVERSNUM) && VDRVERSNUM < 10300
-    dsyslog("CHANNELOUT: Switching bitstream thread started (pid=%d)\n", getpid());
-# endif
+#if defined(VDRVERSNUM) && VDRVERSNUM < 10300
+    dsyslog("CHANNELOUT: Switching bitstream thread started (pid=%d)", getpid());
+#endif
 
     ctrl.Lock();
     while (test_flag(ACTIVE)) {
 	uint_16 apid;
 	const char* type;
 
-	watch.TimedWait(ctrl, 200);			// 200ms
-
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-	if ((tracktimeout -= 200) <= 0)
-	    break;
-#endif
-	if (test_setup(CLEAR))
-	    break;
+	watch.TimedWait(ctrl, 100);			// 100ms
 
 	if (!test_flag(ACTIVE))
 	    break;
 
-	if (!in || !Channel)
+	if (test_setup(CLEAR) || !test_setup(ACTIVE))
+	    break;
+
+	if (replaying || recording)
+	    continue;
+
+	if (!Channel)
 	    continue;
 
 	if (!GetCurrentAudioTrack(apid, type))		// Get audio pid and type
 	    continue;
 
-#if defined(VDRVERSNUM) && VDRVERSNUM < 10321
+#if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
 	if (strcmp(type, audioType) != 0)		// The green button switch
 	    continue;					// changes languages only
 #endif
@@ -831,9 +913,9 @@ void cChannelOutSPDif::Action(void)
     }
     ctrl.Unlock();
 
-# if defined(VDRVERSNUM) && VDRVERSNUM < 10300
-    dsyslog("CHANNELOUT: Switching bitstream thread ended (pid=%d)\n", getpid());
-# endif
+#if defined(VDRVERSNUM) && VDRVERSNUM < 10300
+    dsyslog("CHANNELOUT: Switching bitstream thread ended (pid=%d)", getpid());
+#endif
     clear_flag(RUNNING);
 }
 
@@ -842,21 +924,20 @@ void cChannelOutSPDif::AudioSwitch(uint_16 apid, const char* type)
     sw.Lock();
 
     if (!test_setup(ACTIVE)) {
-	if (test_flag(ACTIVE)) Activate(false);
+	Activate(false);
 	if (in) AttachReceiver(false);
+	Channel = NULL;			// Reset Channel
 	goto out;
-    }
-#if defined(VDRVERSNUM) && VDRVERSNUM < 10321
-    else {
+    } else {
 	if (!test_flag(RUNNING)) {
 	    clear_flag(ACTIVE);
 	    Activate(true);
 	}
     }
-#endif
-    if (in) AttachReceiver(false);
 
-    if (replaying) {
+    if (in) AttachReceiver(false);	// Do _not_ reset Channel
+
+    if (replaying || recording) {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	goto out;	
 #else
@@ -894,8 +975,7 @@ void cChannelOutSPDif::AttachReceiver(bool onoff)
 	onoff = false;
 
     if (in) {
-	if (PrimaryDevice)
-	    PrimaryDevice->Detach(in);
+	in->Clear();
 	delete in;
 	in = NULL;
 	bounce->flush();
@@ -935,15 +1015,7 @@ void cChannelOutSPDif::ChannelSwitch(const cDevice *Device, int channelNumber)
     if (!Device->IsPrimaryDevice())
 	goto out;
 
-    // Sanity check not be attached in transfer mode.
-    // In this case the replay part should take the data
-    if (PrimaryDevice != cDevice::ActualDevice())
-	goto out;
-
-    // Next sanity check not be attached in transfer mode
-    // from the primary device to the primary device.
-    // As metioned above the replay part should take the data
-    if (PrimaryDevice == cTransferControl::ReceiverDevice())
+    if (InTransferMode())
 	goto out;
 
     // If we're noticed only for channel switch we're done.
@@ -953,9 +1025,10 @@ void cChannelOutSPDif::ChannelSwitch(const cDevice *Device, int channelNumber)
 	// Problem: Channel switches for recording on the same
 	// transponder will cause a small audio pause.
 	//
-	if (in)
-	    AttachReceiver(false);
+	sw.Lock();
+	if (in) AttachReceiver(false);
 	Channel = NULL;			// Reset Channel
+	sw.Unlock();
 	goto out;
     }
 
@@ -977,24 +1050,46 @@ out:
     return;
 }
 
+void cChannelOutSPDif::Recording(const cDevice  *Device,  const char *Name)
+{
+    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
+
+    if (!PrimaryDevice)
+	goto out;
+
+    if (Device != PrimaryDevice)
+	goto out;
+
+    recording = (Name != NULL);
+
+    if (!recording)
+	goto out;
+
+    sw.Lock();
+    if (in) AttachReceiver(false);
+    Channel = NULL;			// Reset Channel
+    sw.Unlock();
+out:
+    IfNeededMuteSPDIF();		// On close the S/P-DIF is not muted anymore
+}
+
 void cChannelOutSPDif::Replaying(const cControl *Control, const char *Name)
 {
     replaying = (Name != NULL);
 
 #if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
-    Player = ((const cControlPerm *)Control)->Player();
+    Player = (static_cast<const cControlPerm *>(Control))->Player();
 #endif
 
-    if (replaying) {
-	if (in)
-	    AttachReceiver(false);
-	Channel = NULL;			// Reset Channel
-    }
-    //else
-    //	Do nothing here. Attaching a receiver will be done by ChannelSwitch.
-    //	Anything else will cause trouble in transfer and replay mode.
-    //
-    IfNeededMuteSPDIF();	// On close the S/P-DIF is not muted anymore
+    if (!replaying)
+	goto out;
+
+    sw.Lock();
+    if (in) AttachReceiver(false);
+    Channel = NULL;			// Reset Channel
+    sw.Unlock();
+out:
+    IfNeededMuteSPDIF();		// On close the S/P-DIF is not muted anymore
 }
 
 void cChannelOutSPDif::IfNeededMuteSPDIF(void)
@@ -1003,9 +1098,12 @@ void cChannelOutSPDif::IfNeededMuteSPDIF(void)
 	char *cmd = NULL;
 	asprintf(&cmd, "%s %s 2> /dev/null", SPDIFmute, test_setup(MUTE) ? "mute" : "unmute");
 	if (cmd) {
-	    if (test_setup(MP2ENABLE))
-		setenv("loop", "off", 1);
-	    else
+	    if (test_setup(ACTIVE)) {
+		if (test_setup(MP2ENABLE))
+		    setenv("loop", "off", 1);
+		else
+		    setenv("loop", "on", 1);
+	    } else
 		setenv("loop", "on", 1);
 	    system(cmd);
 	    free(cmd);
@@ -1018,12 +1116,15 @@ bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type)
 {
     bool ret = false;
 
-    if (replaying)
+    if (replaying || recording)
 	goto out;
 
     if (!Channel)
 	goto out;
 
+    if (InTransferMode())
+	goto out;
+
     ret = GetCurrentAudioTrack(apid, type, Channel);
 out:
     return ret;
@@ -1031,12 +1132,13 @@ out:
 
 bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type, cChannel *channel)
 {
-    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
+    cDevicePerm *PrimaryDevice = static_cast<cDevicePerm *>(cDevice::PrimaryDevice());
     eTrackType CurrentAudioTrack = ttNone;
     const tTrackId *track;
     int num;
 #else
+    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
     uint_16 mpid = 0x1FFF;
     uint_16 dpid = 0x1FFF;
     int CurrentAudioTrack = 0;
@@ -1058,6 +1160,8 @@ bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type, cC
 
     for (num = 0; num < MAXAPIDS; num++) {
 	if (channel->Apid(num) == track->id) {
+	    if (!PrimaryDevice->UsePid(track->id))
+		goto out;
 	    apid = channel->Apid(num);
 	    type = audioTypes[IEC_MP2];
 	    break;
@@ -1107,7 +1211,7 @@ out:
 
 bool cChannelOutSPDif::AudioTrack(int num, const char* type, int &apid)
 {
-    if (replaying) {
+    if (replaying || recording) {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	apid = 0x1FFF;
 #else
@@ -1149,7 +1253,7 @@ out:
 
 uint_16 cChannelOutSPDif::AudioPid(void)
 {
-    if (replaying) {
+    if (replaying || recording) {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	return 1;
 #else
@@ -1167,9 +1271,16 @@ uint_16 cChannelOutSPDif::AudioPid(void)
 
 const char* cChannelOutSPDif::AudioType(void)
 {
-    if (replaying)
+    if (replaying || recording)
 	return audioTypes[IEC_NONE];
     if (in)
 	return in->AudioType();
     return audioType;
 }
+
+void cChannelOutSPDif::Clear(void)
+{
+    Activate(false);
+    if (in) AttachReceiver(false);
+    Channel = NULL;
+}
