@@ -61,6 +61,9 @@ cReplayOutSPDif::cReplayOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const c
 
 cReplayOutSPDif::~cReplayOutSPDif(void)
 {
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10320
+    if (test_flag(ACTIVE))
+#endif
     Clear();
 }
 
@@ -106,7 +109,6 @@ void cReplayOutSPDif::Activate(bool onoff)
 	    iec60958* curr = stream;
 	    ctr.Unlock();
 	    if (curr) {				// thread broken
-		LOCK_THREAD;
 		spdifDev->Close();
 		clear_flag(BOUNDARY);
 		curr->Reset();
@@ -137,8 +139,7 @@ void cReplayOutSPDif::Action(void)
 	set_flag(FAILED);
 	goto out;
     } else {
-	LOCK_THREAD;
-        if (!spdifDev->Open(curr)) {
+        if (!spdifDev->Open(curr, this)) {
 	    esyslog("REPLAY: can't open spdif interface");
 	    set_flag(FAILED);
 	    goto out;
@@ -154,7 +155,6 @@ void cReplayOutSPDif::Action(void)
 
 	if (test_setup(MUTE)) {
 	    if (!test_flag(WASMUTED)) {
-		LOCK_THREAD;
 		spdifDev->Clear();
 		clear_flag(BOUNDARY);
 		bounce->flush();
@@ -164,14 +164,13 @@ void cReplayOutSPDif::Action(void)
 	}
 
 	if (!spdifDev->Synchronize(bounce))	// Wait on bounce->signal()
-	    break;
+	    continue;				// No break on large delay gaps
 
 	if (!test_flag(ACTIVE))
 	    break;
 
 	if (test_setup(MUTE)) {
 	    if (!test_flag(WASMUTED)) {
-		LOCK_THREAD;
 		spdifDev->Clear();
 		clear_flag(BOUNDARY);
 		bounce->flush();
@@ -185,11 +184,9 @@ void cReplayOutSPDif::Action(void)
 	    spdifDev->Forward(pesdata, len, bounce);
 
     }
-    Lock();
-    spdifDev->Close();
+    spdifDev->Close(this);
     clear_flag(BOUNDARY);
     curr->Reset();
-    Unlock();
 #if defined(VDRVERSNUM) && VDRVERSNUM < 10300
     dsyslog("REPLAY: Forwarding bitstream thread ended (pid=%d)", getpid());
 #endif
@@ -218,10 +215,11 @@ inline bool cReplayOutSPDif::OffsetToDvdOfPS1(const uchar *const b, int &off, co
     TEST(dvd >= (size_t)4) {
 	uint_32 ul = dvd;
 	uint_8  ub = (uint_8)(ul>>24);
-	uint_8  tr = (ub & 7) + 1;
-#if 0
+	uint_8  tr = (ub & 0x1F) + 0x21;
 	off_t   o  = (off_t) (ul & 0x0000ffff) + 3;
-#endif
+
+	unsigned int sample_rate;
+	uint_8 m;
 
 	switch (ub) {
 	case 0x80 ... 0x87:			// AC3 from e.g. DVD
@@ -245,15 +243,18 @@ inline bool cReplayOutSPDif::OffsetToDvdOfPS1(const uchar *const b, int &off, co
 	    off += 4;
 	    break;
 	case 0xa0 ... 0xa7:			// Linear PCM Audio
-	    if (curr != &pcm) {
+	    if (o == 3) {			// broken stream
 		dvd++;
 		uint_32 ui = dvd;
 		dvd += 4;
-		uint_8  m  = dvd;
+		m = dvd;
 		if ((ui == 0) && (m == 0)) {
 		    off = cnt;
 		    break;			// Don't support broken streams (dvd plugin).
 		}
+	    } else {
+		dvd += 5;
+		m = dvd;
 	    }
 	    if (curr != &pcm || curr->track != tr)
 		break;
@@ -261,6 +262,19 @@ inline bool cReplayOutSPDif::OffsetToDvdOfPS1(const uchar *const b, int &off, co
 	    if (id != ub)
 		break;
 #endif
+	    switch (m & 0x30) {
+	    case 0x00: sample_rate = 48000;
+		break;
+	    case 0x20: sample_rate = 44100;
+		break;
+	    case 0x30: sample_rate = 32000;
+		break;
+	    case 0x10:				// 96kHz  currently not supported
+	    default:   sample_rate = 0;
+		break;
+	    }
+	    if (curr->sample_rate != sample_rate)
+		break;
 	    if (rest < 7)
 		break;
 	    ret = true;				// found, success
@@ -295,6 +309,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 	// byte2, byte3 (offset to frame which corresponds to PTS value)
 	uint_32 ul = dvb;
 	uint_8  ub = (uint_8)(ul>>24);
+	uint_8  tr = (ub & 0x1F) + 0x21;
 	off_t   o  = (off_t) (ul & 0x0000ffff);
 
 	bool isDVD = false;
@@ -320,7 +335,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 	    clear_setup(AUDIO);
 	    curr->Reset(setup.flags);
 	    curr->isDVD = isDVD;
-	    curr->track = ((isDVD) ? (ub&7)+1 : 0x0b);
+	    curr->track = ((isDVD) ? tr : 0x21);
 	    goto out;				// found, go out
 	case 0x88 ... 0x8f:			// DTS
 	    if (o == 0) o = 1;			// broken audio sub header
@@ -341,7 +356,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 	    clear_setup(AUDIO);
 	    curr->Reset(setup.flags);
 	    curr->isDVD = isDVD;
-	    curr->track = ((isDVD) ? (ub&7)+1 : 0x7f);
+	    curr->track = ((isDVD) ? tr : 0x21);
 	    goto out;				// found, go out
 	case 0xa0 ... 0xa7:			// Linear PCM Audio
 	// byte4	(emphasis, mute, reserved, 4-0: frame number modulo 20)
@@ -371,7 +386,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 	    set_setup(AUDIO);
 	    curr->Reset(setup.flags);
 	    curr->isDVD = true;
-	    curr->track = (ub&7)+1;
+	    curr->track = tr;
 
 	    switch (m & 0x30) {
 	    case 0x00: curr->sample_rate = 48000;
@@ -409,7 +424,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 		    clear_setup(AUDIO);
 		    curr->Reset(setup.flags);
 		    curr->isDVD = false;
-		    curr->track = 0x0b;
+		    curr->track = 0x21;
 		    break;
 		}
 		if ((dvb > (size_t)4) && (uint_32)dvb == DTSmagic) {
@@ -417,7 +432,7 @@ inline bool cReplayOutSPDif::ScanPayOfPS1(const uchar *const b, int &off, const
 		    clear_setup(AUDIO);
 		    curr->Reset(setup.flags);
 		    curr->isDVD = false;
-		    curr->track = 0x7f;
+		    curr->track = 0x21;
 		    break;
 		}
 		dvb++;
@@ -548,8 +563,11 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 
 	switch (mag) {
 	case PS1magic:				// Audio of PES Private Stream 1
-	    if (curr == &mp2)			// MP2 already active
+
+	    if (curr == &mp2) {			// MP2 already active
+		Clear();			// Reset
 		goto out;
+	    }
 
 	    if (!test_flag(BOUNDARY)) {
 	    
@@ -561,11 +579,17 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 		if ((flg & 0x00C0) || (flg & 0x0400)) {
 
 #if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-		    if (!ScanPayOfPS1(&b[0], off, cnt, id) || !curr)
+		    if (!ScanPayOfPS1(&b[0], off, cnt, id))
 #else
-		    if (!ScanPayOfPS1(&b[0], off, cnt) || !curr)
+		    if (!ScanPayOfPS1(&b[0], off, cnt))
 #endif
 			goto out;
+		    
+		    ctr.Lock();
+		    curr = stream;
+		    ctr.Unlock();
+		    if (!curr) goto out;	// Paranoid
+
 		    set_flag(BOUNDARY);		// At the beginning of a stream
 
 		    if (!test_flag(RUNNING)) {	// Start or continue after mute the
@@ -582,27 +606,50 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 		} else
 		    goto out;			// Start only at a boundary of a stream
 
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
+	    } else if (curr) {
+
+		if (curr->isDVD) {
+		    // Check if DVD stream is still valid
+		    if (!OffsetToDvdOfPS1(b, off, cnt, id)) {
+			Clear();		// Reset
+			goto out;
+		    }
+		} else {
+		    // Check if id of the stream is still valid
+		    if (id != 0xBD) {
+			Clear();		// Reset
+			goto out;
+		    }
+		}
+	    }
+#else
 	    } else if (curr && curr->isDVD) {
 
 		// Check if DVD stream is still valid
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-		if (!OffsetToDvdOfPS1(b, off, cnt, id))
-#else
-		if (!OffsetToDvdOfPS1(b, off, cnt))
-#endif
-		{
+		if (!OffsetToDvdOfPS1(b, off, cnt)) {
 		    Clear();			// Reset
 		    goto out;
 		}
-
 	    }
+#endif
 	    break;
 	case 0x000001c0 ... 0x000001df:		// Audio of PES Mpeg Audio frame
 	    if (!test_setup(MP2ENABLE))
 		goto out;
 
-	    if (curr && curr != &mp2)	// PS1 already active
-		goto out;
+	    if (curr) {
+		uint_8 track = ((uint_8)(mag & 0xFF) - 0xC0) + 1;
+
+		if (curr != &mp2) {		// PS1 already active
+		    Clear();			// Reset
+		    goto out;
+		}
+		if (curr->track != track) {	// Wrong track
+		    Clear();			// Reset
+		    goto out;
+		}
+	    }
 
 	    if (test_flag(BOUNDARY))
 		break;
@@ -611,11 +658,17 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 		goto out;
 
 #if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
-	    if (!DigestPayOfMP2(&b[0], off, cnt, id) || !curr)
+	    if (!DigestPayOfMP2(&b[0], off, cnt, id))
 #else
-	    if (!DigestPayOfMP2(&b[0], off, cnt) || !curr)
+	    if (!DigestPayOfMP2(&b[0], off, cnt))
 #endif
 		goto out;
+
+	    ctr.Lock();
+	    curr = stream;
+	    ctr.Unlock();
+	    if (!curr) goto out;		// Paranoid
+
 	    set_flag(BOUNDARY);			// At the beginning of a stream
 
 	    if (!test_flag(RUNNING)) {		// Start or continue after mute the
@@ -629,7 +682,7 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 		clear_flag(WASMUTED);
 	    }
 
-	    curr->track = (uint_8)(mag & 7) + 1;
+	    curr->track = ((uint_8)(mag & 0xFF) - 0xC0) + 1;
 
 	    break;
 	default:
@@ -703,9 +756,12 @@ out:
 	char *cmd = NULL;
 	asprintf(&cmd, "%s %s 2> /dev/null", SPDIFmute, test_setup(MUTE) ? "mute" : "unmute");
 	if (cmd) {
-	    if (test_setup(MP2ENABLE) && test_setup(ACTIVE))
-		setenv("loop", "off", 1);
-	    else
+	    if (test_setup(ACTIVE)) {
+		if (test_setup(MP2ENABLE))
+		    setenv("loop", "off", 1);
+		else
+		    setenv("loop", "on", 1);
+	    } else
 		setenv("loop", "on", 1);
 	    system(cmd);
 	    free(cmd);
@@ -720,7 +776,7 @@ void cReplayOutSPDif::Clear(void)
     cDevice *PrimaryDevice = cDevice::PrimaryDevice();
 
     debug("cReplayOutSPDif::Clear() called\n");
-    (void)Activate(false);
+    Activate(false);
     ctr.Lock();
     iec60958* curr = stream;
     ctr.Unlock();
