@@ -81,12 +81,28 @@ void cPsleep::usec(const int us)
     pthread_mutex_unlock(&mutex);
 }
 
+// --- cHold : Be able to lock/unlock within a thread without cThreadLock --------------
+
+bool cHold::Hold(cThread *Thread)
+{
+    if (thl) return true;
+    thl = new cThreadLock(Thread);
+    return (thl) ? true : false;
+}
+
+void cHold::Unhold(void)
+{
+    if (!thl) return;
+    delete thl;
+    thl = (cThreadLock*)0;
+}
+
 // --- spdif : VDR interface for opening S/P-DIF interface of sound cards --------------
 
 uint_32 spdif::silent_buf[PCM_SILENT_10MS48KHZ];
 
 spdif::spdif(ctrl_t &up)
-: thread(NULL), setup(up), wait()
+: cHold(), thread(NULL), setup(up), wait()
 {
     out = NULL;
     stream = NULL;
@@ -150,10 +166,18 @@ xout:
     return (stream != NULL);
 }
 
+inline bool spdif::Frame(frame_t &pcm,
+			 const uint_8 *&head, const uint_8 *const tail)
+{
+    HOLD(thread);
+    return (pcm = stream->Frame(head, tail)).burst != NULL;
+}
+
 //
 // Forward the incoming data to S/P-DIF
 //
-void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *bounce)
+void spdif::Forward(const uint_8 *const data,
+		    const size_t dlen, class cBounce *bounce)
 {
     const uint_8 *      head = data;
     const uint_8 *const tail = data + dlen;
@@ -164,7 +188,7 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 	goto xout;
 
     ctrlbits &= ~FL_IO;
-    while ((pcm = stream->Frame(head, tail)).burst) {
+    while (Frame(pcm, head, tail)) {
 
 	if ((ctrlbits & FL_NOEXSYNC) || (ctrlbits & FL_IO))
 	    check();
@@ -186,7 +210,6 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 		//
 		if (!stream->pts.dvbcheck(duration))
 		    continue;
-		count = 10;
 
 		// Ignore differences with more than +/- 1000 (accordingly DVB specs)
 		offset = stream->pts.delay(1000);
@@ -203,51 +226,67 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 		    pause = 0;
 		}
 
+		Hold(thread);			// Hold the lock on the calling thread
+
 		//
-		// Time offset for AC3 data, do this _before_ sending PCM_WAIT
+		// Time offset to real linear and nonlinear PCM data
 		//
-		if (!opt.audio) {
+		{
 		    register int ddelay = offset/10;
 
+		    if (opt.audio) ddelay += opt.adelay;
+
 		    for (int n = 0; n < ddelay; n++) {	// Every burst is 10 ms silent
-			burst(silent);
-			if (check() == SPDIF_HIGH) {
+			switch (check()) {
+			case SPDIF_HIGH:
+			    Unhold();		// Do not hold lock on calling thread
 			    EINTR_RETRY(snd_pcm_wait(out, 10));
+			    // fall through
+			case SPDIF_OK:
+			default:
+			    burst(silent);
+			    break;
 			}
 		    }
 		}
 
+		Hold(thread);			// Hold the lock on the calling thread
+
 		//
 		// Start frame, PCM_WAIT frames seems to be ignored by decoders
 		// for linear PCM we use PCM_WAIT2 to get the input buffer for
 		// linear PCM of the AV receiver free.
 		//
 		const frame_t init = stream->Frame(((opt.audio) ? PCM_WAIT2 : PCM_WAIT));
-		mcnt = (test_setup(LIVE) ? opt.mdelay : 2);
+
+		if (test_setup(LIVE)) {
+		    mcnt  = opt.mdelay;
+		    count = (mcnt < 7) ? 10 : mcnt + 4;
+		} else {
+		    mcnt  = 2;
+		    count = 10;
+		}
+
 		do {
-		    burst(init);
-		    count--;
-		} while ((check() < SPDIF_OK) && (count > mcnt));
+		    switch (check()) {
+		    case SPDIF_HIGH:
+			// fall through
+		    case SPDIF_OK:
+			break;
+		    default:
+			burst(init);
+			count--;
+			break;
+		    }
+		} while (count > mcnt);
+
+		if (opt.audio) count += 5;
+
+		Unhold();			// Do not hold lock on calling thread
 
 		if (check() == SPDIF_HIGH)
 		    EINTR_RETRY(snd_pcm_wait(out, 10));
 
-		//
-		// Time offset for PCM data, do this _after_ sending PCM_WAIT
-		//
-		if (opt.audio) {
-		    register int ddelay = offset/10;
-
-		    ddelay += opt.adelay;
-
-		    for (int n = 0; n < ddelay; n++) {	// Every burst is 10 ms silent
-			burst(silent);
-			if (check() == SPDIF_HIGH) {
-			    EINTR_RETRY(snd_pcm_wait(out, 10));
-			}
-		    }
-		}
-
 		ctrlbits &= ~(FL_FIRST|FL_UNDERRUN);
 		repeat = 0;
 	    }
@@ -259,6 +298,8 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 		//
 		if (ctrlbits & FL_VRPERIOD) {
 
+		    Hold(thread);		// Hold the lock on the calling thread
+
 		    if (opt.audio) {
 #if 0
 			// We're playing real PCM data, therefore we play
@@ -272,7 +313,9 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 			if (!(repeat = (++repeat) % 4)) {
 			    // Use a wait frame for filling
 			    const frame_t fill = stream->Frame(PCM_WAIT2);
+			    // stream->SetErr();
 			    burst(fill);
+			    // stream->ClearErr();
 			}
 #endif
 		    } else {
@@ -286,6 +329,8 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 			}
 		    }
 
+		    Unhold();			// Do not hold lock on calling thread
+
 		    // FL_VRPERIOD will be removed by external call of
 		    // Synchronize() check()ing the buffers state
 
@@ -310,6 +355,7 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 
 		// Just avoid that burst() is hanging around.
 		if (check() == SPDIF_HIGH) {
+		    Unhold();			// Do not hold lock on calling thread
 		    if (skip)
 			// This avoids extrem overruns
 			EINTR_RETRY(snd_pcm_wait(out, period));
@@ -335,9 +381,11 @@ void spdif::Forward(const uint_8 *const data, const size_t dlen, class cBounce *
 
 	if (count > 0) {
 	    //
-	    // At least 10 start frames should go around
+	    // At least 10 start frames should go around for nonlinear PCM
 	    //
-	    pcm = opt.audio ? stream->Frame(PCM_SILENT) : stream->Frame(PCM_WAIT);
+	    const unsigned int size = pcm.size;
+	    pcm = stream->Frame(((opt.audio) ? PCM_SILENT : PCM_WAIT));
+	    pcm.size = size;
 	    count--;
 	}
 
@@ -366,6 +414,9 @@ void spdif::burst(const frame_t &pcm)
     while((frames > 0) && out) {
 	snd_pcm_sframes_t res = 0;
 	if ((res = writei(out, (const void *)data, frames)) < 0) {
+
+	    Unhold();			// Do not hold lock on calling thread
+
 	    switch(res) {
 	    case -EBUSY:
 		EINTR_RETRY(snd_pcm_wait(out, 10));
@@ -403,6 +454,7 @@ void spdif::burst(const frame_t &pcm)
 	}
 
 	if (res < (snd_pcm_sframes_t)fragsize) {
+	    Unhold();			// Do not hold lock on calling thread
 	    EINTR_RETRY(snd_pcm_wait(out, -1));
 	}
 
@@ -420,7 +472,7 @@ xout:
 //
 // Scan options and open the S/P-DIF device
 // 
-bool spdif::Open(iec60958 *in)
+bool spdif::Open(iec60958 *in, cThread *caller)
 {
     char pcm_name[256];
     snd_pcm_access_t access;
@@ -432,10 +484,13 @@ bool spdif::Open(iec60958 *in)
 	Close();
 
     ctrlbits = FL_FIRST|FL_NOEXSYNC;
+    thread = caller;
 
     if (!Stream(in))
 	goto err_null;
 
+    Hold(thread);	// Hold lock on calling thread
+
     opt.card   = setup.opt.card;
     opt.device = setup.opt.device;
     if (test_setup(LIVE)) {
@@ -744,7 +799,7 @@ __diga_end:
 	    buffer_size = err;
 	    tenth = buffer_size/10;
 	    buf.upper = buffer_size - 3*tenth;
-	    buf.lower = 4*tenth;
+	    buf.lower = (opt.audio ? 2 : 4)*tenth;
 	    buf.high  = buffer_size - tenth;
 	    buf.alarm = tenth/3;
 	}
@@ -824,6 +879,7 @@ __diga_end:
 	goto err_status;
     }
 
+    Unhold();
     Unlock();
     return true;
 
@@ -839,6 +895,7 @@ err_null:
     status = NULL;
     esyslog("S/P-DIF: unable to establish BitStreamOut for none audio PCM\n");
 
+    Unhold();
     Unlock();
     return false;
 }
@@ -846,7 +903,7 @@ err_null:
 //
 // Close the S/P-DIF device
 // 
-void spdif::Close(void)
+void spdif::Close(cThread *caller)
 {
     bool exit = true;
     snd_pcm_t *tmp = out;
@@ -855,9 +912,12 @@ void spdif::Close(void)
     if (!out)
         goto err;
 
+    Unhold();					// Leave any thead lock if any
     while (ctrlbits & FL_BURSTRUN)
 	wait.msec(1);
+    if (caller) Hold(caller);			// Hold lock for pause or stop frame
     Clear(exit);
+    Unhold();					// leave holded lock
 
     thread = NULL;
     out    = NULL;
@@ -901,7 +961,7 @@ void spdif::Clear(bool exit)
     if (!out)
 	goto xout;
 
-    ctrlbits |= FL_PAUSE;		// Mute all
+    ctrlbits |= FL_PAUSE;			// Mute all
 
     if (ctrlbits & FL_FIRST)
 	goto xout;
@@ -916,7 +976,7 @@ void spdif::Clear(bool exit)
 
 	if (exit) {
 	    ctrlbits &= ~FL_PAUSE;
-	    pause = 0;			// VDR has cleared its buffers
+	    pause = 0;				// VDR has cleared its buffers
 
 	    if (stream) {
 		// Say decoder to wait for pause or stop
@@ -924,6 +984,8 @@ void spdif::Clear(bool exit)
 		burst(stop);
 	    }
 
+	    Unhold();				// Do not hold lock on calling thread
+
 	    if (opt.mmap) {
 		// In case of mmap access we've to wait
 		delay = snd_pcm_avail_update(out);
@@ -945,6 +1007,9 @@ void spdif::Clear(bool exit)
 		}
 	    }
 	} else {
+
+	    Unhold();				// Leave any thead lock if any
+
 	    //
 	    // In Replay mode VDR does _not_ empty its buffers for Pause
 	    //
@@ -1008,6 +1073,8 @@ bool spdif::Synchronize(class cBounce *bounce)
     if (ctrlbits & FL_FIRST)
 	goto do_wait;
 
+    Unhold();				// Leave any thead lock if any
+
     ctrlbits &= ~FL_NOEXSYNC;
 
 repeat:
@@ -1111,6 +1178,9 @@ inline void spdif::xunderrun(void)
 
     if (!out)
 	goto xout;
+
+    Unhold();				// Leave any thead lock if any
+
     if ((res = snd_pcm_status(out, status))<0) {
 	esyslog("S/P-DIF: status error: %s", snd_strerror(res));
 	goto xout;
@@ -1143,6 +1213,9 @@ inline void spdif::xsuspend (void)
 
     if (!out)
 	goto xout;
+
+    Unhold();				// Leave any thead lock if any
+
     if ((res = snd_pcm_status(out, status))<0) {
 	esyslog("S/P-DIF: status error: %s", snd_strerror(res));
 	goto xout;
