@@ -886,7 +886,7 @@ static off_t scan_c0(uint_8* const buf, const off_t rest, const uint_16 flags, c
 
     static sint_32 len[8];
     cHandle handle(buf, rest);
-    bool aligned = (flags & 0x0400) || (flags & 0x00C0);
+    bool aligned = (flags & 0x0400);
 
     if (id >= 8)
 	goto out;
@@ -956,15 +956,32 @@ static off_t scan_c0(uint_8* const buf, const off_t rest, const uint_16 flags, c
 
 	int padding = (ub & 0x02) >> 1;
 
+	uint_16 size;
 	uint_16 bytes;
 	if (layer == 1) {
-	    bytes = 4*((12 * bitrate / rate) + padding);
+	    size  = 4*((12 * bitrate / rate) + padding);
+	    bytes = 1536;
 	} else {
 	    const uint_16 slots = ((layer == 3) && lsf) ? 72 : 144;
-	    bytes = (slots * bitrate / rate) + padding;
+	    size  = (slots * bitrate / rate) + padding;
+	    bytes = 4608;
 	}
 
-	uint_64 lpts = ((2*bytes)*90*1000)/rate;
+	uint_64 lpts;
+	switch (rate) {
+	case 48000:
+	    lpts = ((15*bytes)/32);
+	    break;
+	case 44100:
+	    lpts = ((75*bytes)/147);
+	    break;
+	case 32000:
+	    lpts = ((45*bytes)/64);
+	    break;
+	default:
+	    lpts = 0;
+	    break;
+	}
 #ifdef AUDIO_CRC
 	uint_16 target = 0;
 	if (crc) {
@@ -974,13 +991,13 @@ static off_t scan_c0(uint_8* const buf, const off_t rest, const uint_16 flags, c
 	}
 #endif
 	nl;
-	N("%8llu FF mpeg audio layer=%d, len=%d", offset, layer, bytes);
-	if ((handle.Offset() - done) + bytes <= (uint_64)rest) {
-	    handle += (bytes - done);
+	N("%8llu FF mpeg audio layer=%d, len=%d", offset, layer, size);
+	if ((handle.Offset() - done) + size <= (uint_64)rest) {
+	    handle += (size - done);
 	} else {
 	    len[id] = (rest - (handle.Offset() - done));
 	    N("(%d)", len[id]);
-	    len[id] = bytes - len[id];
+	    len[id] = size - len[id];
 	    stop = true;
 	}
 	N(", dur=%llums", lpts/90);
@@ -1029,7 +1046,8 @@ static off_t scan_bd(uint_8* const buf, const off_t rest, const uint_16 flags, c
 	{ 640 ,{1280 ,1393 ,1920 } }, { 640 ,{1280 ,1394 ,1920 } }
     };
 
-    bool aligned = (flags & 0x0400) || (flags & 0x00C0);
+    bool aligned = (flags & 0x0400);
+    bool haspts  = (flags & 0x00C0);
 
     if (plainac3)			// Likely a singular DVB AC3 stream
 	goto nosub;
@@ -1049,9 +1067,10 @@ static off_t scan_bd(uint_8* const buf, const off_t rest, const uint_16 flags, c
 	    handle++;
 	    {
 		static sint_32 len = 0;
-		if (!aligned && !len)
+		if (!aligned && !haspts && !len)
 		    break;
-		if (aligned) {
+		if (aligned) len = 0;
+		if ((len == 0) || haspts) {
 		    substream = true;
 		    len = (uint_16)handle;
 		    nl;
@@ -1101,9 +1120,10 @@ static off_t scan_bd(uint_8* const buf, const off_t rest, const uint_16 flags, c
 		break;
 	    {
 		static sint_32 len = 0;
-		if (!aligned && !len)
+		if (!aligned && !haspts && !len)
 		    break;
-		if (aligned) {
+		if (aligned) len = 0;
+		if ((len == 0) || haspts) {
 		    substream = true;
 		    len = (rest - o);
 		    nl;
@@ -1138,41 +1158,40 @@ nosub:
 	static sint_32 len = 0;		// Set if DVB stream with not aligned AC3 frames in payload
 	    sint_32 l = rest;
 	    uint_16 m;
-	    if (aligned || len) {
-		if (len) {
-		    off_t off = handle.Offset();
-		    handle += len;
-		    nl;			// If not on boundary a overroll may occured
-		    N("%8llu   ", deep+off);
-		    N(" plain(AC3) rest=%d", len);
-		    l -= len;
-		    len = 0;
-		}
-		while (l > 0) {
-		    m = handle;		// May throw an Exeption if no AC3 frame follows
-		    if (m == 0x0b77) {
-			uint_16 fscod, frmsizecod;
-			handle += 4;
-			plainac3 = true;
+	    if (aligned) len = 0;
+	    if (len) {
+		off_t off = handle.Offset();
+		handle += len;
+		nl;			// If not on boundary a overroll may occured
+		N("%8llu   ", deep+off);
+		N(" plain(AC3) rest=%d", len);
+		l -= len;
+		len = 0;
+	    }
+	    while (l > 0) {
+		m = handle;		// May throw an Exeption if no AC3 frame follows
+		if (m == 0x0b77) {
+		    uint_16 fscod, frmsizecod;
+		    handle += 4;
+		    plainac3 = true;
 
-			fscod	   = (((uint_8)handle)>>6)&3;
-			frmsizecod =  ((uint_8)handle)&0x3f;
-			len = 2*frmsizecod_tbl[frmsizecod].frame_size[fscod];
+		    fscod	= (((uint_8)handle)>>6)&3;
+		    frmsizecod	=  ((uint_8)handle)&0x3f;
+		    len = 2*frmsizecod_tbl[frmsizecod].frame_size[fscod];
 
-			nl;
-			N("%8llu 0B", deep+handle.Offset()-4);
-			N(" plain(AC3) size=%d", len);
-			if (l < len) {
-			    N(" included=%u", l);
-			    len -= l;
-			    break;
-			}
-			l -= len;
-			handle += (len - 4);
-		    } else {
-			handle++;	// Scan for initial AC3 frame for not aligned payload
-			l--;
+		    nl;
+		    N("%8llu 0B", deep+handle.Offset()-4);
+		    N(" plain(AC3) size=%d", len);
+		    if (l < len) {
+			N(" included=%u", l);
+			len -= l;
+			break;
 		    }
+		    l -= len;
+		    handle += (len - 4);
+		} else {
+		    handle++;		// Scan for initial AC3 frame for not aligned payload
+		    l--;
 		}
 	    }
 	} end (handle);
